/* Copyright (c) 2009 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */

/**
 * Control object for the notes view.
 *
 * @type Object
 */
var tbNotesNotesView =
{
  
  /**
   * Returns main Thunderbird's window - we needed it because of access 
   * to the tbNotes.
   * @private
   * @returns {Components.interfaces.nsIDOMWindow}
   */
  getMsgWindow : function()
  {
    var winMan = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
    var winManInt = winMan.QueryInterface(Components.interfaces.nsIWindowMediator);
    var win = winManInt.getMostRecentWindow("mail:3pane");
    
    return win;
  }, // end getMsgWindow()
  
  /**
   * Edit note with given ID
   *
   * @param aNoteId {integer}
   * @returns {void}
   */
  editNote : function(aNoteId)
  {
    with(this.getMsgWindow()) {
      tbNotes.showEditNoteDialog(aNoteId, false);
    }
    // TODO Reload only if any note was really updated!
    window.location.reload();
  }, // end editNote(aNoteId)
  
  /**
   * Delete note with given ID
   *
   * @param aNoteId {integer}
   * @returns {boolean}
   */
  deleteNote : function(aNoteId)
  {
    with(this.getMsgWindow()) {
      tbNotesDb.deleteNote(aNoteId);
    }
    // TODO Reload only if any note was really deleted!
    window.location.reload();
  }, // end deleteNote(aNoteId)
  
  /**
   * Delete selected notes
   *
   * @returns {void}
   */
  deleteSelectedNotes : function()
  {
    $(".singlenote").each(function(index, item) {
      var note_id = (new String(item.getAttribute("id"))).replace("note-", "");
      
      if(document.getElementById("id-check-" + note_id).checked) {
        with(tbNotesNotesView.getMsgWindow()) {
          tbNotesDb.deleteNote(note_id);
        }
        
        update = true;
      }
    });
    
    // TODO Update only when any note was really deleted!
    window.location.reload();
  }, // end deleteSelectedNotes()
  
  /**
   * Opens target of note with given ID
   *
   * @param aNoteId {integer}
   * @returns {boolean}
   */
  openTarget : function(aNoteId)
  {
    alert(aNoteId + ":" + aTargetType + ":" + aTargetKey);
  }, // end openTarget(aNoteId)
  
  /**
   * Returns object with array of notes that match current filter and count
   * of all notes.
   *
   * @returns {object} E.g. { filtered: [NotePrototype, ...], totalCount: 12 }
   */
  getNotes : function()
  {
    var notes = new Array();
    var total = 0;
    
    with(this.getMsgWindow()) {
      // TODO Get notes according to current filter!
      notes = tbNotesDb.getNotes();
      total = notes.length;
    }
    
    return { filtered: notes, totalCount: total };
  }, // end getNotes()
  
  /**
   * Show text of note with given ID.
   *
   * @param aNoteId {integer}
   * @returns {void}
   */
  showNoteText : function(aNoteId)
  {
    $("#note_" + aNoteId + "_showLink-span").hide();
    $("#note_" + aNoteId + "_hideLink-span").show();
    $("#note_" + aNoteId + "_body-div").show();
  }, // end showNoteText(aNoteId)
  
  /**
   * Hide text of note with given ID.
   *
   * @param aNoteId {integer}
   * @returns {void}
   */
  hideNoteText : function(aNoteId)
  {
    $("#note_" + aNoteId + "_showLink-span").show();
    $("#note_" + aNoteId + "_hideLink-span").hide();
    $("#note_" + aNoteId + "_body-div").hide();
  }, // end hideNoteText(aNoteId)
  
  /**
   * Returns preferences for the notes view.
   *
   * @returns {object}
   */
  getPrefs : function()
  {
    var viewPrefs = {}
    var onceAgain = false
    
    try {
      var prefs = Components.classes["@mozilla.org/preferences-service;1"].
          getService(Components.interfaces.nsIPrefBranch);
      
      viewPrefs = {
        filter: {
          deleted: prefs.getBoolPref("extensions.tbnotes.notesview.filter.deleted"),
          updated: prefs.getBoolPref("extensions.tbnotes.notesview.filter.updated"),
          to_mail: prefs.getBoolPref("extensions.tbnotes.notesview.filter.to_mail"),
          to_address: prefs.getBoolPref("extensions.tbnotes.notesview.filter.to_address"),
          to_newsitem: prefs.getBoolPref("extensions.tbnotes.notesview.filter.to_newsitem"),
          to_feeditem: prefs.getBoolPref("extensions.tbnotes.notesview.filter.to_feeditem"),
          notarget: prefs.getBoolPref("extensions.tbnotes.notesview.filter.notarget")
        },
        show_texts: prefs.getBoolPref("extensions.tbnotes.notesview.show_texts")
      };
    } catch(e) {
      // If there were any errors is commonly means that preferences are not saved yet so do it
      prefs.setBoolPref("extensions.tbnotes.notesview.filter.deleted", true);
      prefs.setBoolPref("extensions.tbnotes.notesview.filter.updated", false);
      prefs.setBoolPref("extensions.tbnotes.notesview.filter.to_mail", true);
      prefs.setBoolPref("extensions.tbnotes.notesview.filter.to_address", true);
      prefs.setBoolPref("extensions.tbnotes.notesview.filter.to_newsitem", true);
      prefs.setBoolPref("extensions.tbnotes.notesview.filter.to_feeditem", true);
      prefs.setBoolPref("extensions.tbnotes.notesview.filter.notarget", true);
      prefs.setBoolPref("extensions.tbnotes.notesview.show_texts", false);
      
      onceAgain = true;
    }
    
    return (onceAgain === true) ? this.getPrefs() : viewPrefs;
  }, // end getPrefs()
  
  /**
   * Updates notes view
   *
   * @returns {void}
   */
  updateNotesView : function()
  {
    var prefs = this.getPrefs();
    var res = this.getNotes();
    var notes = res.filtered;
    
    // Update sidebar
    this.updateSidebar(notes.length, res.totalCount, prefs);
    
    // Update notes view
    this.updateNotesTable(notes, res.totalCount, prefs.show_texts);
  }, // end updateNotesView()
  
  /**
   * Updates notes view sidebar
   *
   * @param aDisplayed {integer}
   * @param aTotal {integer}
   * @param aOptions {object}
   * @returns {void}
   */
  updateSidebar : function(aDisplayed, aTotal, aOptions)
  {
    $("#displayed-notes-count").text(aDisplayed);
    $("#total-notes-count").text(aTotal);
    
    if(aOptions.filter.deleted == true) {
      $("#filter-deleted-check").attr("selected","true");
    }
    if(aOptions.filter.deleted == true) {
      $("#filter-deleted-check").attr("selected","true");
    }
    if(aOptions.filter.to_mail == true) {
      $("#filter-to_mail-check").attr("checked", "checked");
    }
    if(aOptions.filter.to_address == true) {
      $("#filter-to_address-check").attr("checked", "checked");
    }
    if(aOptions.filter.to_newsitem == true) {
      $("#filter-to_newsitem-check").attr("checked", "checked");
    }
    if(aOptions.filter.to_feeditem == true) {
      $("#filter-to_feeditem-check").attr("checked", "checked");
    }
    if(aOptions.filter.notarget == true) {
      $("#filter-notarget-check").attr("checked", "checked");
    }
    
    if(aOptions.show_texts == true) {
      $("#option-show_texts-check").attr("checked", "checked");
    }
  }, // end updateNotesTable(aDisplayed, aTotal, aOptions)
  
  /**
   * Updates notes table
   *
   * @param aNotes {array}
   * @param aTotalCount {integer}
   * @param aShowTexts {boolean}
   * @returns {void}
   */
  updateNotesTable : function(aNotes, aTotalCount, aShowTexts)
  {
    $("#notesview").html("");
    
    // Stringbundles are loaded from `tbNotes.properties`
    try {
      var messenger = tbNotesNotesView.getMsgWindow();
      var msg1 = messenger.tbNotes.mStrbundle.getString("noNotesMsg1");
      var msg2 = messenger.tbNotes.mStrbundle.getString("noNotesMsg2");
      var lbl1 = messenger.tbNotes.mStrbundle.getString("noTarget");
      var lbl2 = messenger.tbNotes.mStrbundle.getString("show");
      var lbl3 = messenger.tbNotes.mStrbundle.getString("hide");
      var lbl4 = messenger.tbNotes.mStrbundle.getString("edit");
      var lbl5 = messenger.tbNotes.mStrbundle.getString("remove");
    } catch(e) {
      Components.utils.reportError(e);
    }
    
    if(aNotes.length == 0) {
      var msg = (aTotalCount == 0) ? msg1 : msg2;
      
      $("#notesview-controls").hide();
      $("#notesview").append(
        '<tr class="singlenote">' +
        '  <th scope="row" class="check" rowspan="2"> </th>' +
        '  <td class="target"> </td>' + 
        '  <td class="updated"> </td>' + 
        '  <td class="title"><p><strong>' + msg + '</strong></p></td>' + 
        '</tr>' +
        '<tr><td colspan="3"> </td></tr>');
      
      return;
    }
    
    // Helper  method for formatting note's date
    var formatDate = function(aDate) {
      var p = aDate.split(" ");
      var d = p[0].split("-");
      //var t = p[1].split(":");
      return (new Number(d[2])) + "." + (new Number(d[1])) + ", " + d[0]/* + " " + 
             (new Number(t[0])) + ":" + (new Number(t[1]))*/;
    };
    
    for(var i=0; i<aNotes.length; i++) {
      var targetClass =
          (aNotes[i].mTargetType == "" || aNotes[i].mTargetType == null)
              ? "notarget"
              : aNotes[i].mTargetType;
      
      var targetAction =
          (targetClass != "notarget" && (aNotes[i].mTargetKey != "" || aNotes[i].mTargetKey != null))
              ? " onclick=\"tbNotesNotesView.openTarget('" + aNotes[i].mId + "', '" + aNotes[i].mTargetType + "', '" + aNotes[i].mTargetKey + "');\""
              : "";
      
      var targetName = (targetClass == "notarget") ? lbl1 : aNotes[i].mTargetName;
      var target = "<span" + targetAction + " class=\"" + targetClass + "\"> </span>";
      
      try {
        $("#notesview").append(
          '<tr id="note-' + aNotes[i].mId + '" class="singlenote">' +
          '  <th scope="row" class="check" rowspan="2"><input type="checkbox" id="id-check-' + aNotes[i].mId + '" class="note-check"></input></th>' +
          '  <td class="target">' + target + '</td>' + 
          '  <td class="updated">' + formatDate(aNotes[i].mUpdated) + '</td>' +
          '  <td class="title">' +
          '    <p>' +
          '      <strong>' + aNotes[i].mTitle + '</strong> (' +
          '<span id="note_' + aNotes[i].mId + '_showLink-span" onclick="tbNotesNotesView.showNoteText(\'' + aNotes[i].mId + '\');" class="link">' + lbl2 + '</span>' +
          '<span id="note_' + aNotes[i].mId + '_hideLink-span" onclick="tbNotesNotesView.hideNoteText(\'' + aNotes[i].mId + '\');" class="link" style="display:none;">' + lbl3 + '</span>' +
          ', <span id="note_' + aNotes[i].mId + '_editLink-span" onclick="tbNotesNotesView.editNote(\'' + aNotes[i].mId + '\');" class="link">' + lbl4 + '</span>' +
          ', <span id="note_' + aNotes[i].mId + '_removeLink-span" onclick="tbNotesNotesView.deleteNote(\'' + aNotes[i].mId + '\');" class="link">' + lbl5 + '</span>)' +
          '    </p>' +
          '  </td>' + 
          '</tr>' +
          '<tr>' +
          '  <td colspan="3">' +
          '    <div id="note_' + aNotes[i].mId + '_body-div" class="notebody" ' + ((aShowTexts == true) ? '' : ' style="display:none;"') + '>' +
          '      <pre>' + aNotes[i].mBody + '</pre>' +
          '    </div>' + 
          '  </td>' + 
          '</tr>');
      } catch(e) {}
    }
  } // end updateNotesTable(aNotes, aTotalCount, aShowTexts)
  
}; // End of tbNotesNotesView

// =========================================================================
// Initialize notes view
$(function() {
  tbNotesNotesView.updateNotesView();
  
  // Check/uncheck all notes
  $("#checkallnotes-checkbox").click(function(aEvent) {
    if(this.checked) {
      $(".note-check").attr("checked", true);
    } else {
      $(".note-check").removeAttr("checked");
    }
  });
  
  // Show/hide full texts
  $("#option-show_texts-check").change(function(aEvent) {
    if(this.checked) { // Ensure that texts are shown
      $("div.notebody").show();
    } else { // Ensure that texts are hidden
      $("div.notebody").hide();
    }
  });
  
});
